/* Main cereal interface
   Copyright (C) 2002, 2004 Miloslav Trmac <mitr@volny.cz>

   You can redistribute and/or modify this program under the terms of the
   GNU General Public License version 2 as published by the Free Software
   Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place Suite 330, Boston, MA 02111-1307, USA. */
#ifndef CEREAL_H__
#define CEREAL_H__

#include <config.h>

#include <stdint.h>

#include <lib.h>

BEGIN_C_DECLS

struct _xmlNode;
struct _xmlNs;

/* Detected conditions in this emulation step (see ER_* below), modified
   by emulation functions */
extern int emulation_result;

enum
{
  ER_HALT	= 1,			/* There is no emulation to be done */
  ER_ERROR	= ER_HALT << 1,		/* Unrecoverable error reported */
  ER_WARNING	= ER_ERROR << 1,	/* Warning reported */
  ER_INSN	= ER_WARNING << 1,	/* One insn emulation was finished */
  ER_BREAKPOINT	= ER_INSN << 1		/* A breakpoint triggered */
};

/* Time format used by the schedule code */
struct schedule_time
{
  unsigned second;
  double subsecond;
};

/* Time, as kept by the schedule code */
extern struct schedule_time cereal_time;

/* Opaque representation of a schedulable event */
struct schedule_event;

/* Pointer to schedule event function */
typedef void (*schedule_fn_t) (void *mi);

/* Schedule emulation FN to be called for EVENT, TIME seconds later */
extern void schedule_add (struct schedule_event *event, double time,
			  schedule_fn_t fn);

/* Cancel a scheduled, not yet activated EVENT */
extern void schedule_cancel (struct schedule_event *event);

/* Create a new schedulable event for use by MI_;
   returns: schedulable event */
extern struct schedule_event *schedule_new (void *mi_);

/* Delete EVENT (canceling it if it is already scheduled) */
extern void schedule_delete (struct schedule_event *event);

/* Check whether there is an event scheduled */
extern int schedule_pending (void);

/* Run next scheduled event;
   returns: emulation result */
extern int schedule_run (void);

/* Return difference between A and B, in seconds */
extern double schedule_difftime (const struct schedule_time *a,
				 const struct schedule_time *b);

/* Schedule function description */
struct schedule_fn_desc
{
  schedule_fn_t fn;
  const char *name;
};

/* Save EVENT state ax XML element NS:NAME under NODE using (NULL-terminated)
   LIST of possible functions */
extern void schedule_save_event (struct schedule_event *event,
				 struct _xmlNode *node, struct _xmlNs *ns,
				 const char *name,
				 const struct schedule_fn_desc *list);

/* Restore EVENT state from NODE of DOC using (NULL-terminated) LIST of
   possible functions;
   returns: 0 if OK, -1 on error */
extern int schedule_load_event (struct schedule_event *event,
				struct _xmlNode *node,
				const struct schedule_fn_desc *list);

/* Save global scheduling state to NODE */
extern void schedule_save_state (struct _xmlNode *node);

/* Restore global scheduling state from NODE of DOC;
   returns: 0 if OK, -1 on error */
extern int schedule_load_state (struct _xmlDoc *doc, struct _xmlNode *node);

END_C_DECLS

#endif
