/* Expression evaluation
   Copyright (C) 2002, 2004 Miloslav Trmac <mitr@volny.cz>

   You can redistribute and/or modify this program under the terms of the
   GNU General Public License version 2 as published by the Free Software
   Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place Suite 330, Boston, MA 02111-1307, USA. */
#ifndef EXPR_H__
#define EXPR_H__

#include <config.h>

#include <stdint.h>

#include <lib.h>

BEGIN_C_DECLS

enum
{
  EC_VALUE	= 0x01,	/* We want to get only the current value */
  EC_MODIFY	= 0x02 /* Don't check that E_PORT ports can be displayed */
};
 
/* Compile EXPR according to FLAGS (see EC_* below);
   returns: expression (for free ()) if OK, NULL on error */
extern struct expr_code *expr_compile (const char *expr, unsigned flags);

/* Evaluate EXPR, store result to DEST;
   returns: 0 if OK, -1 on error */
extern int expr_eval (const char *expr, intmax_t *dest);

/* Evaluate compiled EXPR, store result to DEST;
   returns: 0 if OK, -1 on error */
extern int expr_exec (const struct expr_code *expr, intmax_t *dest);

/* Modify port referenced in DEST so that DEST == SRC;
   returns: 0 if OK, -1 on error */
extern int expr_modify (const char *dest, const char *src);

END_C_DECLS

#endif
