/* Module instances management
   Copyright (C) 2002, 2004 Miloslav Trmac <mitr@volny.cz>

   You can redistribute and/or modify this program under the terms of the
   GNU General Public License version 2 as published by the Free Software
   Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place Suite 330, Boston, MA 02111-1307, USA. */
#ifndef ME_H__
#define ME_H__

#include <config.h>

#include <lib.h>

BEGIN_C_DECLS;

struct _xmlDoc;
struct _xmlNode;

/* Known module instance */
struct me_entry
{
  struct me_entry *next; /* Change only using me_new () etc. */
  char *name;
  const struct cereal_module *module;
  void *mi;
};

/* List of all me's */
extern struct me_entry *me_list;

/* Find me NAME;
   returns: me if found, NULL otherwise */
extern struct me_entry *me_find (const char *name);

/* Find me NAME, report error if not found;
   returns: me if found, NULL otherwise */
extern struct me_entry *me_get (const char *name);

/* Check if NAME is OK
   returns: 1 if it is, 0 otherwise */
extern int me_name_ok (const char *name);

/* Create new me for MODULE called NAME;
   returns: me if OK, NULL on error */
extern struct me_entry *me_new (const struct cereal_module *module,
				const char *name);

/* Rename me MODULE to NEW_NAME;
   returns: 0 if OK, -1 on error */
extern int me_rename (const char *old, const char *new_name);

/* Delete me NAME */
extern void me_delete (const char *name);

/* Delete all known me's */
extern void me_delete_all (void);

/* Save setup (configuration options) of all me's as children of NODE */
extern void me_save_setup (struct _xmlNode *node);

/* Load setup (configuration options of me's according to NODE of DOC;
   returns: 0 if OK, -1 on error */
extern int me_load_setup (struct _xmlDoc *doc, struct _xmlNode *node);

/* Save state of all me's as children of NODE */
extern void me_save_state (struct _xmlNode *node);

/* Load state of me's according to NODE of DOC;
   returns: 0 if OK, -1 on error */
extern int me_load_state (struct _xmlDoc *doc, struct _xmlNode *node);

END_C_DECLS;

#endif
