/* Cereal port value access interface
   Copyright (C) 2002, 2004, 2005 Miloslav Trmac <mitr@volny.cz>

   You can redistribute and/or modify this program under the terms of the
   GNU General Public License version 2 as published by the Free Software
   Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place Suite 330, Boston, MA 02111-1307, USA. */
#ifndef PORT_H__
#define PORT_H__

#include <config.h>

#include <stddef.h>
#include <stdint.h>

#include <lib.h>
#include <module.h>

BEGIN_C_DECLS

struct me_entry;
struct space;
struct _xmlDoc;
struct _xmlNode;

/* Human-readable port type names */
extern const char *const port_type_name[NUM_PORT_TYPES];

/* Port identification */
struct port_id
{
  const struct me_entry *me;
  size_t space, port;
};

/* Active port connections list */
struct connect_entry
{
  struct connect_entry *next, **pprev; /* Don't modify directly! */
  int type;
  enum port_width width;
  struct port_id pi1, pi2;
};

/* List of current port connections */
extern struct connect_entry *connect_list;

/* Read WIDTH (NUM_WIDTHS = any) port specification from SPEC, store to *PI;
   returns: 0 if OK, -1 on error */
extern unsigned port_find (enum port_width width, const char *spec,
			   struct port_id *pi);

#define TYPE__(WIDTH, TYPE)						   \
/* Get value of port *PI, store it to DEST;				   \
   returns: 0 if OK, -1 on error */					   \
extern int port_disp_##WIDTH (const struct port_id *pi, TYPE *dest);	   \
									   \
/* Set port *PI to VAL;							   \
   returns: 0 if OK, -1 on error */					   \
extern int port_set_##WIDTH (const struct port_id *pi, unsigned long val);

TYPE__ (1, int);
TYPE__ (8, uint8_t);
TYPE__ (16, uint16_t);

#undef TYPE__

/* Create port name for PI of WIDTH;
   returns: port name from libxml (in UTF-8) (for xmlFree ()) */
extern xmlChar *port_create_name (enum port_width width,
				  const struct port_id *pi);

/* Read port number from NAME (in UTF-8), store to PI of WIDTH;
   returns: 0 if OK, -1 on error (PI->port not modified) */
extern int port_find_name (enum port_width width, struct port_id *pi,
			   const char *name);

/* Resolve port type from NAME;
   returns: port type or -1 if not found */
extern int port_type_find (const char *name);

/* Find connection going "the other way" (read / write) than CE;
   returns: peer connection or NULL if not found */
extern struct connect_entry *port_peer_connection (const struct connect_entry
						   *ce);

/* Create a new connection using CE;
   returns: CE if OK, NULL on error */
struct connect_entry *port_do_connect (struct connect_entry *ce);

/* Create a new connection according to data in *SKEL;
   returns: connection entry if OK, NULL on error */
extern struct connect_entry *port_connect (const struct connect_entry *skel);

/* Disconnect CE without deallocating it */
extern void port_do_disconnect (struct connect_entry *ce);

/* Disconnect CE */
extern void port_disconnect (struct connect_entry *ce);

/* Delete all connections involving ME */
extern void port_disconnect_me (const struct me_entry *me);

/* Forget about all known port connections */
extern void port_disconnect_all (void);

/* Save list of all known port connectios as children of NODE */
extern void port_connections_save (struct _xmlNode *node);

/* Load port connectios according to NODE of DOC;
   returns: 0 if OK, -1 on error */
extern int port_connections_load (struct _xmlDoc *doc, struct _xmlNode *node);

END_C_DECLS

#endif
