/* "Setup" ("hardware configuration", modules, their instances and options)
   handling
   Copyright (C) 2002, 2004 Miloslav Trmac <mitr@volny.cz>

   You can redistribute and/or modify this program under the terms of the
   GNU General Public License version 2 as published by the Free Software
   Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place Suite 330, Boston, MA 02111-1307, USA. */
#ifndef SETUP_H__
#define SETUP_H__

#include <config.h>

#include <stdint.h>

#include <lib.h>

BEGIN_C_DECLS

struct _xmlNode;

/* Create an "inital" setup */
extern void setup_init (void);

/* Delete current setup */
extern void setup_destroy (void);

/* Save current setup to FILE;
   returns: 0 if OK, -1 on error */
extern int setup_save (const char *file);

/* Load current setup from FILE;
   returns: 0 if OK, -1 on error (current setup is already destroyed) */
extern int setup_load (const char *file);

/* Save current state to FILE, UI state is saved by FN with DATA;
   returns: 0 if OK, -=1 on error */
extern int state_save (const char *file,
		       void (*fn) (struct _xmlNode *node, intptr_t data),
		       intptr_t data);

/* Load current state from FILE (or create empty state if FILE is
   cereal_setup document), UI state is loaded by UI_FN with DATA,
   call SETUP_FN with DATA when setup is loaded;
   returns: 1 if loaded cereal_state, 0 if loaded cereal_setup, -1 on error
   (current state is already destroyed) */
extern int state_load (const char *file, void (*setup_fn) (intptr_t data),
		       int (*fn) (struct _xmlDoc *doc, struct _xmlNode *node,
				  intptr_t data), intptr_t data);

/* Called when setup is loaded;
   Do nothing */
extern void generic_state_load_setup_fn (intptr_t data);

END_C_DECLS

#endif
