/* Bit value widget
   Copyright (C) 2002, 2004 Miloslav Trmac <mitr@volny.cz>

   You can redistribute and/or modify this program under the terms of the
   GNU General Public License version 2 as published by the Free Software
   Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place Suite 330, Boston, MA 02111-1307, USA. */
#include <cassert>
#include <climits>
#include <cstdio>
#include <cstdlib>
#include <qlabel.h>

#include <expr.h>

#include "kclineedit.h"
#include "kcmainwin.h"
#include "kcbitedit.h"

KCBitEdit::KCBitEdit (QWidget *parent, const char* name)
  : KCPortEditBase (parent, name), compiled (NULL), mainWin (0),
    modifying (false)
{
  setBit ("ERR", "0", 0);
  setFocusProxy (portText);
}

KCBitEdit::~KCBitEdit ()
{
  free (compiled);
}

/** Set control to bit of expr with name */
void
KCBitEdit::setBit (const QString &name, const QString &expr, unsigned bit)
{
  if (compiled != NULL)
    editAcceptChanges ();
  intmax_t val;
  struct expr_code *e = expr_compile (expr, 0);
  if (e == NULL || expr_exec (e, &val) != 0)
    {
      free (e);
      return;
    }
  portLabel->setText (name);
  portLabel->setBuddy (portText);
  expr_ = expr;
  free (compiled);
  compiled = e;
  bit_ = bit;
  cerealUpdate ();
}
  
/** Update view because emulation state has changed */
void
KCBitEdit::cerealUpdate ()
{
  /* FIXME: highlight changed values */
  intmax_t val;
  if (expr_exec (compiled, &val) == 0)
    portText->setText (QString::number ((int)((val & (1 << bit_)) != 0)));
}

/** Set KCMainWin to task to update all views to win */
void
KCBitEdit::setMainWin (KCMainWin *win)
{
  mainWin = win;
}

/** Try to modify the port according to user's change */
void
KCBitEdit::editAcceptChanges ()
{
  if (!portText->edited () || modifying == true)
    return;
  modifying = true;
  intmax_t val, req;
  if (expr_exec (compiled, &val) == 0 && expr_eval (portText->text (), &req)
      == 0)
    {
      if (req < 0 || req >= 2)
	error (0, "expression cannot have the desired value");
      else
	{
	  val = (val & ~(1 << bit_)) | req << bit_;
	  char buf[64]; /* Overkill */
	  assert (sizeof (val) * CHAR_BIT <= 128);
	  /* Do it by hand, because Qt has too few types accepted in
	     QString::number () */
	  sprintf (buf, "%jd", val);
	  expr_modify (expr_, buf);
	}
    }
  cerealUpdate ();
  if (mainWin != 0)
    mainWin->updateViews ();
  modifying = false;
}

/** Edit gained focus */
void
KCBitEdit::editFocusGained ()
{
  /* This is used only in KCPortEdit */
}

#include "kcbitedit.moc"
