/* Breakpoint window item
   Copyright (C) 2002 Miloslav Trmac <mitr@volny.cz>

   You can redistribute and/or modify this program under the terms of the
   GNU General Public License version 2 as published by the Free Software
   Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place Suite 330, Boston, MA 02111-1307, USA. */

#include <cassert>
#include <cstdlib>
#include <klistview.h>
#include <qstring.h>

#include <breakpoint.h>
#include <expr.h>

#include "kcbreakitem.h"

KCBreakItem::KCBreakItem (QListView *parent, const QString &expression)
  : KListViewItem (parent), triggered (false)
{
  bp = breakpoint_new ("1"); /* So that we can't fail */
  assert (bp != NULL);
  setExpression (expression);
}

KCBreakItem::~KCBreakItem ()
{
  breakpoint_delete (bp);
}

QString
KCBreakItem::text (int column) const
{
  switch (column)
    {
    case 0:
      return bp->expr;

    case 1:
      return triggered ? "X" : "";

    default:
      return "";
    }
}

/** Change breakpoint expression, compile it and compute current value */
void
KCBreakItem::setExpression (const QString &expression)
{
  intmax_t val;

  if (expr_eval (expression, &val) != 0)
    return;
  breakpoint_change (bp, expression);
  cerealUpdate ();
}

/** @returns breakpoint expression */
QString
KCBreakItem::expression () const
{
  return bp->expr;
}

/** Recompute value and possibly repaint */
void
KCBreakItem::cerealUpdate ()
{
  bool t;

  t = breakpoint_eval (bp) != 0;
  if (triggered != t)
    {
      triggered = t;
      repaint ();
    }
}
