/* Breakpoint window
   Copyright (C) 2002, 2004 Miloslav Trmac <mitr@volny.cz>

   You can redistribute and/or modify this program under the terms of the
   GNU General Public License version 2 as published by the Free Software
   Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place Suite 330, Boston, MA 02111-1307, USA. */

#include <kaction.h>
#include <klocale.h>
#include <kxmlguifactory.h>
#include <libxml/tree.h>
#include <qpopupmenu.h>
#include <qstringlist.h>
#include <qwidget.h>

#include "kcexprdialog.h"
#include "kcbreakitem.h"
#include "kcbreakwin.h"
#include "kclistview.h"
#include "kcmainwin.h"
#include "kcuistatehandler.h"

unsigned KCBreakWin::title_index; /* = 0; */

KCBreakWin::KCBreakWin (KCMainWin *mainWin, QWidget *parent, const char *name)
  : KCListView (parent, name), KCWindow (this, 0, QString::null, mainWin),
    KXMLGUIBuilder (this)
{
  setWFlags (getWFlags () | Qt::WDestructiveClose);
  
  addColumn (i18n ("Expression"));
  addColumn (i18n ("Triggered"));
  setCaption (i18n ("Breakpoints %1").arg (++title_index));

  actionCollection()->setWidget (this);
  new KAction (i18n ("&Insert..."), 0, this, SLOT (insertBreak ()),
	       actionCollection (), "breakinsert");
  deleteAction = new KAction (i18n ("&Delete"), 0, this, SLOT (deleteBreak ()),
			      actionCollection (), "breakdelete");
  deleteAction->setEnabled (false);
  propertiesAction = new KAction (i18n ("&Properties..."), 0, this,
				  SLOT (editBreak ()), actionCollection (),
				  "breakproperties");
  propertiesAction->setEnabled (false);
  factory = new KXMLGUIFactory (this, this, "guifactory");
  setXMLFile ("breakwinui.rc");
  factory->addClient (this);

  QObject::connect (this, SIGNAL (insertPressed ()), this,
		    SLOT (insertBreak ()));
  QObject::connect (this, SIGNAL (deletePressed ()), this,
		    SLOT (deleteBreak ()));
  QObject::connect (this, SIGNAL (returnPressed (QListViewItem *)), this,
		    SLOT (editBreak ()));
  QObject::connect (this, SIGNAL (doubleClicked (QListViewItem *)), this,
		    SLOT (editBreak ()));
  QObject::connect (this, SIGNAL (selectionChanged ()), this,
		    SLOT (resetBreakActions ()));
  QObject::connect (this, SIGNAL (focusGained ()), this,
		    SLOT (resetBreakActions ()));
  QObject::connect (this, SIGNAL (focusLost ()), this,
		    SLOT (resetBreakActions ()));
  QObject::connect (this, SIGNAL (contextMenu (KListView *, QListViewItem *,
					       const QPoint &)), this,
		    SLOT (showContextMenu (KListView *, QListViewItem *,
					   const QPoint &)));
  resetBreakActions ();
}

KCBreakWin::~KCBreakWin ()
{
}

/** Update view because emulation state has changed */
void
KCBreakWin::cerealUpdate ()
{
  for (QListViewItem *i = firstChild (); i != 0; i = i->nextSibling ())
    (static_cast <KCBreakItem *> (i))->cerealUpdate ();
}

/** Save current UI state under uiState */
void
KCBreakWin::saveState (xmlNodePtr uiState)
{
  QString data("");
  for (QListViewItem *i = firstChild (); i != 0; i = i->nextSibling ())
    data += (static_cast <KCBreakItem *> (i))->expression () + '\n';
  KCUIStateHandler::saveWidget (this, "KCBreakWin", uiState, data.utf8 ());
}

/** Load UI state from node of doc;
    returns: 0 if OK, -1 on error */
int
KCBreakWin::loadState (xmlDocPtr doc, xmlNodePtr node)
{
  if (KCUIStateHandler::loadWidget (this, doc, node) != 0)
    return -1;
  xmlChar *str = xmlNodeListGetString (doc, node->children, 1);
  if (str == NULL)
    return 0;
  QStringList l = QStringList::split ("\n",
				      QString::fromUtf8 ((const char *)str));
  xmlFree (str);
  for (QStringList::Iterator i = l.begin (); i != l.end (); i++)
    new KCBreakItem (this, *i);
  return 0;
}

/** Insert new watch */
void
KCBreakWin::insertBreak ()
{
  KCExprDialog *dlg = new KCExprDialog (this, i18n ("Add Breakpoint"), "");
  if (dlg->exec () == QDialog::Accepted)
    new KCBreakItem (this, dlg->expression ());
  delete dlg;
}

/** Delete currently selected watch */
void
KCBreakWin::deleteBreak ()
{
  QListViewItem *i = selectedItem ();
  if (i != 0)
    delete (static_cast <KCBreakItem *> (i));
}

/** Edit currently selected watch */
void
KCBreakWin::editBreak ()
{
  QListViewItem *i = selectedItem ();
  if (i != 0)
    {
      KCBreakItem *item = static_cast <KCBreakItem *> (i);
      KCExprDialog *dlg = new KCExprDialog (this, i18n ("Modify Breakpoint"),
					    item->expression ());
      if (dlg->exec () == QDialog::Accepted)
	item->setExpression (dlg->expression ());
      delete dlg;
    }
}

/** Enable/disable watch actions */
void
KCBreakWin::resetBreakActions ()
{
  bool enable = parentWidget ()->focusWidget () == this
    && selectedItem () != 0;
  deleteAction->setEnabled (enable);
  propertiesAction->setEnabled (enable);
}

/** Show context menu */
void
KCBreakWin::showContextMenu (KListView *, QListViewItem *, const QPoint &p)
{
  QPopupMenu * menu = static_cast <QPopupMenu *>
    (factory->container ("break_menu", this));
  menu->exec (p);
}

#include "kcbreakwin.moc"
