/* Data/Evaluate/Modify dialog
   Copyright (C) 2002, 2004 Miloslav Trmac <mitr@volny.cz>

   You can redistribute and/or modify this program under the terms of the
   GNU General Public License version 2 as published by the Free Software
   Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place Suite 330, Boston, MA 02111-1307, USA. */

#include <cassert>
#include <climits>
#include <cstdio>
#include <klineedit.h>
#include <klocale.h>
#include <kpushbutton.h>
#include <kstdguiitem.h>
#include <stdint.h>

#include <expr.h>

#include "kcevalmodifydialog.h"
#include "kcexprlineedit.h"
#include "kcmainwin.h"
#include "kde.h"

KCEvalModifyDialog::KCEvalModifyDialog (QWidget *parent, KCMainWin *mainWin)
  : KCEvalModifyDialogBase (parent), mainWin_ (mainWin)
{
  closeButton->setGuiItem (KStdGuiItem::close ());
  modifyButton->setEnabled (false);
}

KCEvalModifyDialog::~KCEvalModifyDialog ()
{
}

/* This is not thread safe! */
static KCEvalModifyDialog *error_dialog; /* = 0; */

static void
evaluate_error_handler (const QString &msg)
{
  assert (error_dialog != 0);
  /* This would under normal OO interfaces not be allowed, but
     KCEvalModifyDialogBase is public ancestor for good reasons (we want the
     object to behave reasonably), which also makes the dialog components
     public. */
  error_dialog->valueText->setText (i18n ("Error: %1").arg (msg));
}

/** exprText has changed */
void
KCEvalModifyDialog::exprChanged ()
{
  QString expr = exprText->text ();
  if (expr.isEmpty ())
    {
      valueText->setText ("");
      return;
    }

  void (*prev_error_handler) (const QString &msg) = error_handler;
  assert (error_dialog == 0);
  /* This check is not thread-safe, so it is not enough.  But it should be
     enough for the unfortunate enough to try to thread this to notice the
     problem. */
  error_dialog = this;
  error_handler = evaluate_error_handler;

  intmax_t value;
  if (expr_eval (expr, &value) == 0)
    {
      char buf[64]; /* Overkill */

      assert (sizeof (value) * CHAR_BIT <= 128);
      /* Do it by hand, because Qt has too few types accepted in
	 QString::number () */
      sprintf (buf, "%jd", value);
      valueText->setText (buf);
    }
  
  error_handler = prev_error_handler;
  error_dialog = 0;
}

/** modifyButton was activated */
void
KCEvalModifyDialog::modifyClicked ()
{
  expr_modify (exprText->text (), newValueText->text ());
  mainWin_->updateViews ();
  exprChanged ();
}

/** newValueText has changed */
void
KCEvalModifyDialog::newValueChanged ()
{
  modifyButton->setEnabled (!newValueText->text ().isEmpty ());
}

#include "kcevalmodifydialog.moc"
