/* Expression input dialog
   Copyright (C) 2002, 2003, 2004 Miloslav Trmac <mitr@volny.cz>

   You can redistribute and/or modify this program under the terms of the
   GNU General Public License version 2 as published by the Free Software
   Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place Suite 330, Boston, MA 02111-1307, USA. */

#include <cassert>
#include <climits>
#include <cstdio>
#include <klineedit.h>
#include <klocale.h>
#include <kpushbutton.h>
#include <kstdguiitem.h>

#include <expr.h>

#include "kcexprdialog.h"
#include "kcexprlineedit.h"
#include "kde.h"

/** Create an "input expression" dialog with given title and initial
    expression */
KCExprDialog::KCExprDialog (QWidget *parent, const QString &title,
			    const QString &expression)
  : KCExprDialogBase (parent)
{
  setCaption (title);
  cancelButton->setGuiItem (KStdGuiItem::cancel ());
  OKButton->setGuiItem (KStdGuiItem::ok ());
  OKButton->setEnabled (false);
  expressionText->setText (expression);
}

KCExprDialog::~KCExprDialog ()
{
}

/** Get current expression value */
QString
KCExprDialog::expression () const
{
  return expressionText->text ();
}

/* This is not thread safe! */
static KCExprDialog *error_dialog; /* = 0; */

static void
evaluate_error_handler (const QString &msg)
{
  assert (error_dialog != 0);
  /* This would under normal OO interfaces not be allowed, but
     KCExprDialogBase is public ancestor for good reasons (we want the object
     to behave reasonably), which also makes the dialog components public. */
  error_dialog->valueText->setText (i18n ("Error: %1").arg (msg));
}

/** expressionText has changed */
void
KCExprDialog::expressionChanged ()
{
  QString expr = expressionText->text ();
  if (expr.isEmpty ())
    {
      valueText->setText ("");
      OKButton->setEnabled (false);
      return;
    }

  void (*prev_error_handler) (const QString &msg) = error_handler;
  assert (error_dialog == 0);
  /* This check is not thread-safe, so it is not enough.  But it should be
     enough for the unfortunate enough to try to thread this to notice the
     problem. */
  error_dialog = this;
  error_handler = evaluate_error_handler;

  intmax_t value;
  if (expr_eval (expr, &value) == 0)
    {
      char buf[64]; /* Overkill */

      assert (sizeof (value) * CHAR_BIT <= 128);
      /* Do it by hand, because Qt has too few types accepted in
	 QString::number () */
      sprintf (buf, "%jd", value);
      valueText->setText (buf);
      OKButton->setEnabled (true);
    }
  else
    OKButton->setEnabled (false);

  error_handler = prev_error_handler;
  error_dialog = 0;
}

#include "kcexprdialog.moc"
