/* Port value widget
   Copyright (C) 2002, 2004 Miloslav Trmac <mitr@volny.cz>

   You can redistribute and/or modify this program under the terms of the
   GNU General Public License version 2 as published by the Free Software
   Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place Suite 330, Boston, MA 02111-1307, USA. */
#include <cassert>
#include <climits>
#include <cstdio>
#include <cstdlib>
#include <qlabel.h>

#include <expr.h>

#include "kclineedit.h"
#include "kcmainwin.h"
#include "kcportedit.h"

KCPortEdit::KCPortEdit (QWidget *parent, const char* name)
  : KCPortEditBase (parent, name), compiled (NULL), mainWin (0),
    modifying (false)
{
  setExpression ("ERR", "0");
  setFocusProxy (portText);
}

KCPortEdit::~KCPortEdit ()
{
  free (compiled);
}

/** Set control to expr with name */
void
KCPortEdit::setExpression (const QString &name, const QString &expr)
{
  if (compiled != NULL)
    editAcceptChanges ();
  intmax_t val;
  struct expr_code *e = expr_compile (expr, 0);
  if (e == NULL || expr_exec (e, &val) != 0)
    {
      free (e);
      return;
    }
  portLabel->setText (name);
  portLabel->setBuddy (portText);
  expr_ = expr;
  free (compiled);
  compiled = e;
  cerealUpdate ();
}
  
/** Update view because emulation state has changed */
void
KCPortEdit::cerealUpdate ()
{
  /* FIXME: highlight changed values */
  intmax_t val;
  if (expr_exec (compiled, &val) == 0)
    {
      char buf[64]; /* Overkill */
      
      assert (sizeof (val) * CHAR_BIT <= 128);
      /* Do it by hand, because Qt has too few types accepted in
	 QString::number () */
      sprintf (buf, "%jd", val);
      portText->setText (buf);
    }
}

/** Set KCMainWin to task to update all views to win */
void
KCPortEdit::setMainWin (KCMainWin *win)
{
  mainWin = win;
}

/** Try to modify the port according to user's change */
void
KCPortEdit::editAcceptChanges ()
{
  if (!portText->edited () || modifying == true)
    return;
  modifying = true;
  expr_modify (expr_, portText->text ());
  cerealUpdate ();
  if (mainWin != 0)
    mainWin->updateViews ();
  modifying = false;
}

/** Edit gained focus */
void
KCPortEdit::editFocusGained ()
{
  emit focusGained (this);
}

#include "kcportedit.moc"
