/* Helper functions for UI state handlers
   Copyright (C) 2002, 2004 Miloslav Trmac <mitr@volny.cz>

   You can redistribute and/or modify this program under the terms of the
   GNU General Public License version 2 as published by the Free Software
   Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place Suite 330, Boston, MA 02111-1307, USA. */

#include <cassert>
#include <climits>
#include <cstdio>
#include <libxml/tree.h>
#include <qwidget.h>

#include "kcuistatehandler.h"

/** Save basic state of widget of class_ under uiState, set element content to
    text if non-NULL;
    returns: node for the widget */
xmlNodePtr
KCUIStateHandler::saveWidget (QWidget *widget, const char *class_,
			      xmlNodePtr uiState, const char *text)
{
  xmlNodePtr node = xmlNewTextChild (uiState, NULL,
				     (const xmlChar *)"ui_window",
				     (const xmlChar *)text);
  xmlSetProp (node, (const xmlChar *)"class", (const xmlChar *)class_);
  QPoint pos = widget->pos ();
  QSize size = widget->size ();
  /* QWorkspace adds a QWorkSpaceChild around each our widget.  Oh well. */
  QWidget *p1 = widget->parentWidget ();
  if (p1 != NULL)
    {
      QWidget *p2 = p1->parentWidget ();
      if (p2 != NULL && p2->inherits ("QWorkspace"))
	pos = p1->pos ();
    }
  char buf[128]; /* Overkill */
  /* Adding the numbers has the widest type of them */
  assert (sizeof (pos.x () + pos.y () + size.width () + size.height ())
	  * CHAR_BIT <= 64);
  sprintf (buf, "%d,%d,%d,%d", pos.x (), pos.y (), size.width (),
	   size.height ());
  xmlSetProp (node, (const xmlChar *)"geometry", (const xmlChar *)buf);
  return node;
}

/** Load state saved by saveWidget to widget from node of doc;
    returns: 0 if OK, -1 on error */
int
KCUIStateHandler::loadWidget (QWidget *widget, xmlDocPtr, xmlNodePtr node)
{
  xmlChar *str = xmlGetProp (node, (const xmlChar *)"geometry");
  if (str == NULL)
    return -1;
  int x, y, width, height, res;
  res = sscanf ((const char *)str, "%d,%d,%d,%d", &x, &y, &width, &height);
  xmlFree (str);
  if (res != 4)
    return -1;
  widget->resize (QSize (width, height));
  widget->move (QPoint (x, y));
  return 0;
}
