/* Abstract base class for cereal UI state handlers -*-c++-*-
   Copyright (C) 2002, 2004 Miloslav Trmac <mitr@volny.cz>

   You can redistribute and/or modify this program under the terms of the
   GNU General Public License version 2 as published by the Free Software
   Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place Suite 330, Boston, MA 02111-1307, USA. */
#ifndef KCUISTATEHANDLER_H__
#define KCUISTATEHANDLER_H__

struct _xmlDoc;
struct _xmlNode;
class QWidget;
class QString;

/** Each KCUIState class handles a subset of the UI state.  After it is
    registered with KCMainWin, the KCMainWin will call its saveState () method
    when saving emulation state and the loadState () method when loading it.
    In addition, KCMainWin calls KCWindow::saveState () for all registered
    views, so the usual KCUIState implementation handles only loading the
    state. */
class KCUIStateHandler
{
public:
  /** Save current UI state under uiState */
  virtual void saveState (struct _xmlNode *uiState) = 0;
  /** Load UI state from uiState of doc;
      returns: 0 if OK, -1 on error */
  virtual int loadState (struct _xmlDoc *doc, struct _xmlNode *uiState) = 0;

  /** Save basic state of widget of class_ under uiState, set element content
      to text if non-NULL;
      returns: node for the widget */
  static struct _xmlNode *saveWidget (QWidget *widget, const char *class_,
				      struct _xmlNode *uiState,
				      const char *text);
  /** Load state saved by saveWidget to widget from node of doc;
      returns: 0 if OK, -1 on error */
  static int loadWidget (QWidget *widget, struct _xmlDoc *doc,
			 struct _xmlNode *node);
};

#endif
