/* 8051 cereal_kde plugin
   Copyright (C) 2002, 2004 Miloslav Trmac <mitr@volny.cz>

   You can redistribute and/or modify this program under the terms of the
   GNU General Public License version 2 as published by the Free Software
   Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place Suite 330, Boston, MA 02111-1307, USA. */

#include <kaction.h>
#include <kfiledialog.h>
#include <klocale.h>
#include <libxml/tree.h>

#include <me.h>
#include <module.h>
#include "../kde/kcmainwin.h"

#include "kc8051.h"
#include "kc8051viewwin.h"

/* Our XML namespace */
extern const char KC8051XMLNs[] = "http://www.fw.cz/mitr/cereal/modules/8051";

KC8051::KC8051 (QObject *parent, const char *name)
  : KParts::Plugin (parent, name)
{
  i8051_module = cereal_module_find ("8051");
  if (i8051_module == NULL)
    return; /* No way. */

  if (!parent->inherits ("KCMainWin"))
    return; /* No way either. */
  mainWin = static_cast <KCMainWin *> (parent);

  new KAction (i18n ("&Load Program..."), 0, this, SLOT (loadProgram ()),
	       actionCollection (), "loadprogram");

  mainWin->registerStateHandler (this);
  mainWin->registerProvider (this);
}

KC8051::~KC8051 ()
{
}

/** Save current UI state under uiState */
void
KC8051::saveState (xmlNodePtr)
{ /* Nothing */
}

/** Load UI state from uiState of doc;
    returns: 0 if OK, -1 on error */
int
KC8051::loadState (xmlDocPtr doc, xmlNodePtr uiState)
{
  int res = 0;
  for (xmlNodePtr node = uiState->children; node != NULL; node = node->next)
    {
      if (node->type != XML_ELEMENT_NODE
	  || strcmp ((const char *)node->name, "ui_window") != 0)
	continue;
      xmlChar *str = xmlGetProp (node, (const xmlChar *)"class");
      if (str == NULL)
	continue;
      if (strcmp ((const char *)str, "KC8051ViewWin") == 0)
	{
	  xmlChar *name = xmlGetNsProp (node, (const xmlChar *)"me",
					(const xmlChar *)KC8051XMLNs);
	  if (name == NULL)
	    {
	      res = -1;
	      goto err;
	    }
	  struct me_entry *me = me_get ((const char *)name);
	  xmlFree (name);
	  if (me == NULL)
	    {
	      res = -1;
	      goto err;
	    }
	  KC8051ViewWin *w = new KC8051ViewWin (this, mainWin, me,
						mainWin->viewParent ());
	  res = w->loadState (doc, node);
	  if (res == 0)
	    w->show ();
	  else
	    delete w;
	}
    err:
      xmlFree (str);
      if (res != 0)
	{
	  error (0, "Invalid 8051 ui_window state");
	  break;
	}
    }
  return res;
}

/** @return list of names of windows that can be created by provideWindow () */
QStringList
KC8051::providedWindowList ()
{
  QStringList res;
  for (const struct me_entry *me = me_list; me != NULL; me = me->next)
    {
      if (me->module == i8051_module)
	res.append (me->name);
    }
  return res;
}

/** Create a window corresponding to member i of the list returned by
    providedWindowList () */
void
KC8051::provideWindow (unsigned i)
{
  for (struct me_entry *me = me_list; me != NULL; me = me->next)
    {
      if (me->module == i8051_module)
	{
	  if (i == 0)
	    {
	      KC8051ViewWin *w = new KC8051ViewWin (this, mainWin, me,
						    mainWin->viewParent ());
	      w->show ();
	      break;
	    }
	  i--;
	}
    }
}

/** 8051/Load Program was selected */
void
KC8051::loadProgram ()
{
  struct me_entry *me = mainWin->selectModule (i8051_module);
  if (me == NULL)
    return;
  QString fn = KFileDialog::getOpenFileName (0, "*.hex", mainWin);
  if (!fn.isEmpty ())
    me->module->set_option (me->mi, "load_hex", fn);
}

#include "kc8051.moc"
