/* KDE setup editor
   Copyright (C) 2002, 2004 Miloslav Trmac <mitr@volny.cz>

   You can redistribute and/or modify this program under the terms of the
   GNU General Public License version 2 as published by the Free Software
   Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place Suite 330, Boston, MA 02111-1307, USA. */
#include <config.h>

#include <cstdarg>
#include <cstdlib>
#include <kaboutdata.h>
#include <kapplication.h>
#include <kcmdlineargs.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <qstring.h>

#include <cereal.h>
#include <setup.h>

#include "khwconf.h"
#include "khwmainwin.h"

/* Report printf-like ERR, append message for ERR_NO if ERR_NO != 0;
   Each frontend defines its own implementation. */
extern "C" void
error (int err_no, const char *err, ...)
{
  char buf[4096]; /* Enough is enough ;-) */
  va_list ap;

  va_start (ap, err);
  vsnprintf (buf, sizeof (buf), err, ap);
  va_end (ap);
  QString msg(buf);
  /* GNU coding standards require appname: message, which is sort of strange
     for dialog boxes. */
  msg[0] = msg[0].upper ();
  if (err_no != 0)
    msg += ": " + QString::fromLocal8Bit (strerror (err_no));
  error_handler (msg);
}

/* Display error message MSG */
static void
default_error_handler (const QString &msg)
{
  KMessageBox::sorry (0, msg, KCmdLineArgs::appName ());
}

/** A function to display error message MSG */
void (*error_handler) (const QString &msg) = default_error_handler;

int
main (int argc, char *argv[])
{
  static const KCmdLineOptions cmdLineOptions[] =
    {
      { "+[Filename]", I18N_NOOP ("Hardware configuration to open"), 0 },
      KCmdLineLastOption
    };
  int result;

  /* OK, I give up.  libmain.a needs modules and modules need libmain.a, so we
     need --{start,end}-group or specifying libmain.a twice on the $(LD)
     command line, or making a libmain.o, none of which is possible using
     libtool.  Let's just pull the needed schedule.o by hand. */
  (void)schedule_pending ();
  KAboutData aboutData ("cereal_khwconf",
			I18N_NOOP ("cereal Hardware Configuration"),
			PACKAGE_VERSION,
			I18N_NOOP ("Cereal emulator hardware configuration "
				   "setup utility"),
			KAboutData::License_GPL_V2,
			"(c) 2002 Miloslav Trmac <mitr@volny.cz>", 0, 0,
			PACKAGE_BUGREPORT);
  aboutData.addAuthor ("Miloslav Trma\xc4\x8d", 0, "mitr@volny.cz",
		       "http://www.fw.cz/mitr");
  
  KCmdLineArgs::init (argc, argv, &aboutData);
  KCmdLineArgs::addCmdLineOptions (cmdLineOptions);

  KApplication app;
  setup_init ();

  if (app.isRestored ())
    {
      RESTORE (KHwMainWin);
    }
  else
    {
      KCmdLineArgs *args = KCmdLineArgs::parsedArgs ();
      if (args->count () > 1)
	KCmdLineArgs::usage (i18n ("Only one parameter expected"));

      KHwMainWin *mainWin =
	new KHwMainWin (args->count () == 1 ? args->arg (0): 0);
      args->clear ();
      mainWin->show ();
    }

  result = app.exec ();
  setup_destroy ();
  return result;
}
