/* Main window of khwconf -*-c++-*-
   Copyright (C) 2002, 2004 Miloslav Trmac <mitr@volny.cz>

   You can redistribute and/or modify this program under the terms of the
   GNU General Public License version 2 as published by the Free Software
   Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place Suite 330, Boston, MA 02111-1307, USA. */
#ifndef KHWMAINWIN_H__
#define KHWMAINWIN_H__

#include <kmainwindow.h>
#include <qstring.h>

class QListViewItem;
class KListView;
class KHwMiList;

/** cereal_hwconf main window.  Note that there may be only one per process,
    because the data is stored in the C backend. */
class KHwMainWin: public KMainWindow
{
  Q_OBJECT

public:
  /** Create a new window, optionally loading FILE if it is non-NULL */
  KHwMainWin (const char *file = 0);
  ~KHwMainWin ();

protected:
  /** @return true if the window can be closed */
  bool queryClose ();
  /** Save settings before window is closed */
  bool queryExit ();
  
private slots:
  /** File/New was selected */
  void fileNew ();
  /** File/Open was selected */ 
  void fileOpen ();
  /** File/Save was selected */
  void fileSave ();
  /** File/Save as was selected */
  void fileSaveAs ();
  /** File/Quit was selected */
  void fileQuit ();
  /** Item/Insert was selected */
  void itemInsert ();
  /** Item/Rename was selected */
  void itemRename ();
  /** Item/Delete was selected */
  void itemDelete ();
  /** Item/Properties was selected */
  void itemProperties ();
  /** Selection of miList has changed */
  void listSelectionChanged ();
  /** MI list item was renamed */
  void listItemRenamed (QListViewItem *item);
  /** MI list requests a context menu */
  void listContextMenu (KListView *l, QListViewItem *i, const QPoint &p);

private:
  /** @returns true if current setup can be overwritten */
  bool canForgetData ();
  /** @returns file name we display to the user */
  QString prettyFilename ();
  /** Update window caption */
  void updateCaption ();
  /** Load setup from file */
  void loadFile (const QString &file);
  
  KHwMiList *miList;
  /** true if the setup has been modified */
  bool modified;
  /** Name of file this configuration was loaded from / saved to, or "" */
  QString filename;
  KAction *renameAction;
  KAction *deleteAction;
  KAction *propertiesAction;
};

#endif
