/* Connection modify dialog -*-c++-*-
   Copyright (C) 2002 Miloslav Trmac <mitr@volny.cz>

   You can redistribute and/or modify this program under the terms of the
   GNU General Public License version 2 as published by the Free Software
   Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place Suite 330, Boston, MA 02111-1307, USA. */
#ifndef KHWMODIFYDIALOG_H__
#define KHWMODIFYDIALOG_H__

#include <knumvalidator.h>

#include <port.h>

#include "khwmodifyconnectiondialogbase.h"

struct me_entry;

class KHwModifyConnectionDialog: public KHwModifyConnectionDialogBase
{
  Q_OBJECT
public:
  /** Create a connection modify dialog for conn_ and possibly conn_peer_ (if
      non-NULL) */
  KHwModifyConnectionDialog (QWidget *parent, struct connect_entry *conn_,
			     struct connect_entry *conn_peer_);
  /** Create a dialog to add a connection from module */
  KHwModifyConnectionDialog (QWidget *parent, struct me_entry *module);
  ~KHwModifyConnectionDialog ();
  /** Is modifyPeerCheckbox checked? */
  bool modifyPeerChecked () const;
  
protected slots:
  /** Port type has changed */
  void typeChanged ();
  /** Port width has changed */
  void widthChanged ();
  /** Source space has changed */
  void sourceSpaceChanged ();
  /** Source port has changed */
  void sourcePortChanged ();
  /** Destination port has changed */
  void destModuleChanged ();
  /** Destination space has changed */
  void destSpaceChanged ();
  /** Destination port has changed */
  void destPortChanged ();
  /** Check if data is OK, and accept () if so */
  void tryAccept ();

private:
  /** The real "constructor", common to both of them */
  void init ();
  struct connect_entry *conn;
  struct connect_entry *conn_peer;
  /** Modify source space list according to other selections, try to preserve
      currently selected item */
  void rebuildSourceSpace ();
  /** Modify source port list according to other selections, try to preserve
      currently selected item */
  void rebuildSourcePort ();
  /** Modify destination space list according to other selections, try to
      preserve currently selected item */
  void rebuildDestSpace ();
  /** Modify destination port list according to other selections, try to
      preserve currently selected item */
  void rebuildDestPort ();
  /** Update state of modifyPeerCheckbox and OKButton according to other
      selections, try to preserve currently selected item */
  void rebuildPeerOK ();
  /** Enable/disable modifyPeerCheckbox, preserving user's last selection
      during the time the checkbox is disabled */
  void enablePeer (bool enable);
  /** Currently we are rebuilding some of the controls, so ignore selection
      changes */
  bool changing;
  /** Currently selected connection parameters from the combobox values */
  struct connect_entry selectedConn;
  /** Last user-selected value of modifyPeerCheckbox (to keep it during the
      time the checkbox is disabled (and unchecked)) */
  bool peer_val;
};

#endif
