/* Intel hex format definitions
   Copyright (C) 2002, 2004 Miloslav Trmac <mitr@volny.cz>

   You can redistribute and/or modify this program under the terms of the
   GNU General Public License version 2 as published by the Free Software
   Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place Suite 330, Boston, MA 02111-1307, USA. */
#ifndef HEX_H__
#define HEX_H__

#include <lib.h>

/* Intel hex (BFD calls it Intel 1) format line layout: Each line starts with
   ":" and ends with "\r\n".  The actual contents are bytes expressed in
   hexadecimal with IH_DATA_LEN bytes after IH_DATA and an appended checksum of
   the whole line. */
enum
{
  IH_DATA_LEN,			/* Number of bytes at IH_DATA */
  IH_ADDR_HI, IH_ADDR_LO,	/* Address - depends on IH_TYPE */
  IH_TYPE,			/* Record type */
  IH_DATA			/* Record-type-specific data starts here */
};

enum
{
  IHT_DATA	/* Put IH_DATA at IH_ADDR */
  /* and others (see bfd/ihex.c) */
};

/* Representation of file and human-readable position (to avoid passing
   many arguments) */
struct file_pos
{
  const char *filename;
  unsigned line;
  const struct file_ops *ops;
  union
  {
    FILE *file;
    const unsigned char *string;
  } v;
};

/* "Stream" generalization */
struct file_ops
{
  /* Get next character from this file;
     returns: next read char or EOF at end of file or error */
  int (*next_char) (struct file_pos *pos);
  /* Was there an I/O error reported? */
  int (*was_error) (struct file_pos *pos);
};

/* Load Intel hex from POS to DATA of LEN;
   returns: 0 if OK, -1 on error */
extern int hex_read_file (struct file_pos *pos, uint8_t *data, size_t len);

#endif
