/* Intel hex format handling
   Copyright (C) 2002, 2004 Miloslav Trmac <mitr@volny.cz>

   You can redistribute and/or modify this program under the terms of the
   GNU General Public License version 2 as published by the Free Software
   Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place Suite 330, Boston, MA 02111-1307, USA. */
#include <config.h>

#include <errno.h>
#include <stdio.h>

#include <lib.h>

#include "hex.h"

/* File operations for <stdio.h> files */
static int
stdio_next_char (struct file_pos *pos)
{
  return fgetc (pos->v.file);
}

static int
stdio_was_error (struct file_pos *pos)
{
  return ferror (pos->v.file);
}

static const struct file_ops stdio_ops =
  { .next_char = stdio_next_char, .was_error = stdio_was_error };

/* Load Intel hex file FILENAME to DATA of LEN;
   returns: 0 if OK, -1 on error */
int
load_hex_file (const char *filename, uint8_t *data, size_t len)
{
  struct file_pos pos;
  int res;

  pos.filename = filename;
  /* Yes, open in binary mode.  We want to read "\r\n" ourselves. */
  pos.v.file = fopen (pos.filename, "rb");
  if (pos.v.file == NULL)
    {
      error (errno, "cannot open `%s'", pos.filename);
      res = -1;
      goto err;
    }
  pos.ops = &stdio_ops;
  res = hex_read_file (&pos, data, len);
  fclose (pos.v.file);
 err:
  return res;
}

/* Load Intel hex FILE (of NAME) to DATA of LEN;
   returns: 0 if OK, -1 on error */
int
load_hex_stdio (const char *name, FILE *file, uint8_t *data, size_t len)
{
  struct file_pos pos;

  pos.filename = name;
  pos.v.file = file;
  pos.ops = &stdio_ops;
  return hex_read_file (&pos, data, len);
}
