/* Intel hex format handling
   Copyright (C) 2002, 2005 Miloslav Trmac <mitr@volny.cz>

   You can redistribute and/or modify this program under the terms of the
   GNU General Public License version 2 as published by the Free Software
   Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place Suite 330, Boston, MA 02111-1307, USA. */
#include <config.h>

#include <libxml/tree.h>

#include <lib.h>

#include "hex.h"

/* File operations for strings */
static int
string_next_char (struct file_pos *pos)
{
  if (*pos->v.string != 0)
    return *pos->v.string++;
  else
    return EOF;
}

static int
string_was_error (struct file_pos *pos)
{
  (void)pos;
  return 0;
}

static const struct file_ops string_ops =
  { .next_char = string_next_char, .was_error = string_was_error };

/* Load Intel hex from XML NODE of DOC to DATA of LEN;
   returns: 0 if OK, -1 on error */
int
load_hex_xml (xmlDocPtr doc, xmlNodePtr node, uint8_t *data, size_t len)
{
  struct file_pos pos;
  xmlChar *str;
  int res;

  str = xmlNodeListGetString (doc, node->children, 1);
  if (str == NULL)
    {
      res = -1;
      goto err;
    }
  pos.filename = "HEX"; /* Dummy label */
  pos.v.string = str;
  pos.ops = &string_ops;
  res = hex_read_file (&pos, data, len);
  xmlFree (str);
 err:
  return res;
}

/* Number of bytes output per one line of save_hex_* output */
enum
{
  SAVE_HEX_PER_LINE = 16
};
