/* save_hex_xml ()
   Copyright (C) 2002 Miloslav Trmac <mitr@volny.cz>

   You can redistribute and/or modify this program under the terms of the
   GNU General Public License version 2 as published by the Free Software
   Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place Suite 330, Boston, MA 02111-1307, USA. */
#include <config.h>

#include <assert.h>
#include <string.h>
#include <libxml/tree.h>

#include <lib.h>

#include "hex.h"

/* Number of bytes output per one line of save_hex_* output */
enum
{
  SAVE_HEX_PER_LINE = 16
};

/* Save DATA of LEN (at most 64K) as Intel hex to XML element NS:NAME under
   NODE */
void
save_hex_xml (xmlNodePtr node, xmlNsPtr ns, const char *name,
	      const uint8_t *data, size_t len)
{
  char *hex, *p;
  size_t base;

  assert (len <= 0x10000);
  /* Counting: per-line overhead for each line, size for the data,
     trailing NUL */
  hex = xmalloc (13 * ((len + SAVE_HEX_PER_LINE - 1) / SAVE_HEX_PER_LINE)
		    + 2 * len + 1);
  base = 0;
  p = hex;
  while (len != 0)
    {
      static const char hex_digits[16] = "0123456789ABCDEF";
      size_t run;
      uint8_t checksum;

      run = SAVE_HEX_PER_LINE;
      if (run > len)
	run = len;
      len -= run;
      *p++ = ':';
      checksum = 0;
#define WRITE_BYTE(VAL)				\
      do					\
	{					\
	  uint8_t t;				\
						\
	  t = (VAL);				\
	  *p = hex_digits[t >> 4];		\
	  p[1] = hex_digits[t & 0x0F];		\
	  p += 2;				\
	  checksum += t;			\
	}					\
      while (0)
      WRITE_BYTE (run);
      WRITE_BYTE (base >> 8);
      WRITE_BYTE (base & 0xFF);
      base += run;
      WRITE_BYTE (IHT_DATA);
      do
	WRITE_BYTE (*data++);
      while (--run != 0);
      WRITE_BYTE ((0x100 - checksum) & 0xFF);
#undef WRITE_BYTE
      *p = '\r';
      p[1] = '\n';
      p += 2;
    }
  *p = 0;
  node = xmlNewTextChild (node, ns, (const xmlChar *)name,
			  (const xmlChar *)hex);
  xmlNodeSetSpacePreserve (node, 1);
  free (hex);
}
