/* Expression code
   Copyright (C) 2002 Miloslav Trmac <mitr@volny.cz>

   You can redistribute and/or modify this program under the terms of the
   GNU General Public License version 2 as published by the Free Software
   Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place Suite 330, Boston, MA 02111-1307, USA. */
#ifndef MAIN_EXPR_H__
#define MAIN_EXPR_H__

#include <stddef.h>
#include <stdint.h>

#include <module.h>
#include <port.h>

/* Welcome to the Kingdom of Forth */

/* One element of expression */
struct expr_code
{
  int code; /* See E_* below; char is enough, but the rest is aligned anyway */
  union
  {
    intmax_t val;
    struct
    {
      enum port_width width;
      struct port_id pi;
    } port;
    struct expr_code *op[3]; /* Operands, only used in expr_modify () */
  } v;
};

enum
{
  E_END,	/* End of expression */
  E_VAL,	/* expr_code.v.val */
  E_PORT,	/* expr_code.v.port */
  E_CPORT,	/* expr_code.v.port, not changed in evaluate/modify */
  E_NEG,	/* -TOS */
  E_NOT,	/* !TOS */
  E_MUL,	/* NOS * TOS */
  E_DIV,	/* NOS / TOS */
  E_MOD,	/* NOS % TOS */
  E_ADD,	/* NOS + TOS */
  E_SUB,	/* NOS - TOS */
  E_LSH,	/* NOS << TOS */
  E_RSH,	/* NOS >> TOS */
  E_LT,		/* NOS < TOS */
  E_GT,		/* NOS > TOS */
  E_LE,		/* NOS <= TOS */
  E_GE,		/* NOS >= TOS */
  E_EQ,		/* NOS == TOS */
  E_NE,		/* NOS != TOS */
  E_BAND,	/* NOS & TOS */
  E_BXOR,	/* NOS ^ TOS */
  E_BOR,	/* NOS | TOS */
  E_LAND,	/* NOS && TOS */
  E_LOR,	/* NOS || TOS */
  E_COND,	/* NNOS ? NOS : TOS */
};

#endif
