/* EDS display/keyboard combo
   Copyright (C) 2002 Miloslav Trmac <mitr@volny.cz>

   You can redistribute and/or modify this program under the terms of the
   GNU General Public License version 2 as published by the Free Software
   Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place Suite 330, Boston, MA 02111-1307, USA. */
#include <config.h>

#include <cassert>
#include <cerrno>
#include <cstdlib>
#include <kled.h>
#include <qcheckbox.h>
#include <libxml/tree.h>

#include <cereal.h>
#include <lib.h>
#include <module.h>

#include "eds_dkdlg.h"

enum
{
  WIDTH = 16
};

struct mis
{
  eds_dkDlg *dlg;
  struct port_fn read[WIDTH], write[WIDTH];
  char p1_val[4];
};

enum
{
  S1_P0		= 0,
  S1_P1		= 8,
  NUM_S1	= WIDTH
};

/* Create a new module instance;
   returns: module instance pointer if OK, NULL on error */
static void *
eds_dk_mi_new (void)
{
  struct mis *mi;
  struct port_fn *pf;
  char *p;
  
  mi = (struct mis *)xmalloc (sizeof (*mi));
  for (pf = mi->read; pf < mi->read + ARRAY_SIZE (mi->read); pf++)
    pf->fn.read_1 = NULL;
  for (pf = mi->write; pf < mi->write + ARRAY_SIZE (mi->write); pf++)
    pf->fn.write_1 = NULL;
  for (p = mi->p1_val; p < mi->p1_val + ARRAY_SIZE (mi->p1_val); p++)
    *p = 1; /* Assume inactive until initialized */
  mi->dlg = new eds_dkDlg ();
  mi->dlg->show ();
  return mi;
}

/* Delete MI_ (may be in any emulation state) */
static void
eds_dk_mi_delete (void *mi_)
{
  struct mis *mi;

  mi = (struct mis *)mi_;
  delete mi->dlg;
  free (mi);
}

/* Save state of MI_ to NODE */
static void
eds_dk_save_state (void *mi_, xmlNodePtr node)
{
  /* FIXME: implement */
  (void)mi_;
  (void)node;
}

/* Load state of MI_ from NODE of DOC;
   returns: 0 if OK, -1 on error */
static int
eds_dk_load_state (void *mi_, xmlDocPtr doc, xmlNodePtr node)
{
  /* FIXME: implement */
  (void)mi_;
  (void)doc;
  (void)node;
  return 0;
}

/* Get number of ports of this space in MI_ */
static size_t 
space_1_get_size (void *mi_)
{
  (void)mi_;
  return NUM_S1;
}

/* Read bit port value in MI_;
   returns: port value */
static int
space_1_display_fn (void *mi_, intptr_t data)
{
  struct mis *mi;
  
  assert ((size_t)data < WIDTH);
  mi = (struct mis *)mi_;
  switch (data)
    {
      /* FIXME: call write callbacks when user changes the button state */
#define DO_BUT(IDX, NAME)						   \
	if (mi->p1_val[IDX] == 0 && mi->dlg->CheckBox##NAME->isChecked ()) \
	  res = 1;
    case 4:
      {
	int res;

	res = 0;
	DO_BUT (0, 4);
	DO_BUT (1, 3);
	DO_BUT (2, 2);
	DO_BUT (3, 1);
	return res;
      }
	
    case 5:
      {
	int res;

	res = 0;
	DO_BUT (0, 5);
	DO_BUT (3, DISP);
	return res;
      }

    default:
      return 1;
#undef DO_BUT
    }
}

/* Save port_fn for PORT reading to DEST */
static void 
space_1_get_read_fn (void *mi_, size_t port, struct port_fn *dest)
{
  assert (port < NUM_S1);
  if (port >= S1_P1 && port < S1_P1 + 8)
    {
      dest->fn.read_1 = space_1_display_fn;
      dest->mi = (struct mis *)mi_;
      dest->data = port - S1_P1;
    }
  else
    dest->fn.read_1 = NULL;
}

/* Write VAL to bit port in MI_ */
static void
space_1_write_fn (void *mi_, intptr_t data, int val)
{
  struct mis *mi;
  KLed::State state;

  assert ((size_t)data < NUM_S1);
  mi = (struct mis *)mi_;
  state = val == 0 ? KLed::On : KLed::Off;
  switch (data)
    {
#define DO_LED(IDX, NUM)				\
      if (mi->p1_val[IDX] == 0)				\
	mi->dlg->KLed##IDX##_##NUM->setState (state);
#define DO_LED2(IDX, NUM)						 \
      if (mi->p1_val[IDX] == 0)						 \
	{								 \
	  mi->dlg->KLed##IDX##_##NUM##a->setState (state);		 \
	  mi->dlg->KLed##IDX##_##NUM##b->setState (state);		 \
	}								
#define DO_BIT(IDX)				\
    case S1_P0 + IDX:				\
      DO_LED2 (0, IDX);				\
      DO_LED2 (1, IDX);				\
      DO_LED2 (2, IDX);				\
      DO_LED2 (3, IDX);				\
      break;
      DO_BIT (0);
      DO_BIT (1);
      DO_BIT (2);
      DO_BIT (3);
      DO_BIT (4);
      DO_BIT (5);
      DO_BIT (6);
    case S1_P0 + 7:
      DO_LED (0, 7);
      DO_LED (1, 7);
      DO_LED (2, 7);
      DO_LED (3, 7);
      break;

    case S1_P1 + 0: case S1_P1 + 1: case S1_P1 + 2: case S1_P1 + 3:
      mi->p1_val[data - S1_P1] = val;
      break;
#undef DO_LED
#undef DO_LED2
#undef DO_BIT
    }
}

/* Save port_fn for PORT writing to DEST */
static void 
space_1_get_write_fn (void *mi_, size_t port, struct port_fn *dest)
{
  assert (port < NUM_S1);
  dest->fn.write_1 = space_1_write_fn;
  dest->mi = (struct mis *)mi_;
  dest->data = port;
}

/* Save port_fn for PORT displaying to DEST */
static void 
space_1_get_display_fn (void *mi_, size_t port, struct port_fn *dest)
{
  assert (port < NUM_S1);
  if (port >= S1_P1 && port < S1_P1 + 8)
    {
      dest->fn.display_1 = space_1_display_fn;
      dest->mi = (struct mis *)mi_;
      dest->data = port - S1_P1;
    }
  else
    dest->fn.display_1 = NULL;
}

/* Set peer read_fn for PORT in MI_ to SRC;
   returns: 0 if OK, errno otherwise */
static int 
space_1_set_read_fn (void *mi_, size_t port, const struct port_fn *src)
{
  struct mis *mi;
  struct port_fn *pf;

  assert (port < NUM_S1);
  mi = (struct mis *)mi_;
  pf = mi->read + port;
  if (pf->fn.read_1 != NULL && src->fn.read_1 != NULL)
    return EBUSY;
  *pf = *src;
  return 0;
}

/* Set peer write_fn for PORT in MI_ to SRC;
   returns: 0 if OK, errno otherwise */
static int 
space_1_set_write_fn (void *mi_, size_t port, const struct port_fn *src)
{
  struct mis *mi;
  struct port_fn *pf;

  assert (port < NUM_S1);
  mi = (struct mis *)mi_;
  pf = mi->write + port;
  if (pf->fn.write_1 != NULL && src->fn.write_1 != NULL)
    return EBUSY;
  *pf = *src;
  return 0;
}

static const struct space eds_dk_space_1 =
{
  space_1_get_size,
  { space_1_get_read_fn, space_1_get_write_fn, space_1_get_display_fn, NULL },
  { space_1_set_read_fn, space_1_set_write_fn, NULL, NULL }
};

static struct cereal_module eds_dk_module =
{
  NULL, NULL, "eds_dk", NULL, NULL, NULL, eds_dk_mi_new, eds_dk_mi_delete,
  generic_module_set_option, generic_module_get_option,
  generic_module_save_setup, generic_module_load_setup, eds_dk_save_state,
  eds_dk_load_state,
  { { &eds_dk_space_1, 1, NULL }, { NULL, 0, NULL }, { NULL, 0, NULL } }
};

extern "C" struct cereal_module *
register_self (void)
{
  return cereal_module_register (&eds_dk_module) == 0 ? &eds_dk_module : NULL;
}
