/* Declarations split out from eds_led.cpp so that admin/am_edit is happy
   Copyright (C) 2004 Miloslav Trmac <mitr@volny.cz>

   You can redistribute and/or modify this program under the terms of the
   GNU General Public License version 2 as published by the Free Software
   Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place Suite 330, Boston, MA 02111-1307, USA. */
#ifndef KCEDS_LED_H__
#define KCEDS_LED_H__

#include <klibloader.h>
#include <kled.h>
#include <kparts/plugin.h>

#include "../kde/kcuistatehandler.h"
#include "../kde/kcwindow.h"
#include "../kde/kcwindowprovider.h"

struct expr_code;
struct me_entry;
class KCMainWin;

class KCEDSLedViewWin: public KLed, public KCWindow
{
  Q_OBJECT
public:
  KCEDSLedViewWin (KCWindowProvider *provider, KCMainWin *mainWin,
		   struct me_entry *me, QWidget *parent = 0,
		   const char *name = 0);
  ~KCEDSLedViewWin ();
  /** Update view because emulation state has changed */
  void cerealUpdate ();
  /** Save current UI state under uiState */
  void saveState (struct _xmlNode *uiState);
  /** Load UI state from node of doc;
      returns: 0 if OK, -1 on error */
  int loadState (struct _xmlDoc *doc, struct _xmlNode *node);

private:
  struct me_entry *me_;
  /** Compiled expressions for @/bit/value */
  struct expr_code *value;
};

class KCEDSLed:
  public KParts::Plugin, private KCUIStateHandler, private KCWindowProvider
{
  Q_OBJECT
public:
  KCEDSLed (QObject *parent = 0, const char *name = 0);
  ~KCEDSLed ();
  /** Save current UI state under uiState */
  void saveState (struct _xmlNode *uiState);
  /** Load UI state from uiState of doc;
      returns: 0 if OK, -1 on error */
  int loadState (struct _xmlDoc *doc, struct _xmlNode *uiState);
  /** @return list of names of windows that can be created by
      provideWindow () */
  QStringList providedWindowList ();
  /** Create a window corresponding to member i of the list returned by
      providedWindowList () */
  void provideWindow (unsigned i);
  
private:
  /** bit_value module */
  const struct cereal_module *bit_value_module;
  /** KCMainWin we are attached to */
  KCMainWin *mainWin;
};

class KCEDSLedFactory: public KLibFactory
{
  Q_OBJECT
public:
  KCEDSLedFactory (QObject *parent = 0, const char *name = 0);
  ~KCEDSLedFactory ();
  QObject *createObject (QObject *parent = 0, const char *name = 0,
			 const char *classname = "QObject",
			 const QStringList &args = QStringList ());

private:
  static KInstance *instance;
};

#endif
