/* Trivial disassembler
   Copyright (C) 2002, 2004 Miloslav Trmac <mitr@volny.cz>

   You can redistribute and/or modify this program under the terms of the
   GNU General Public License version 2 as published by the Free Software
   Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place Suite 330, Boston, MA 02111-1307, USA. */
#include <config.h>

#include <inttypes.h>
#include <stdarg.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <disasm8051.h>
#include <lib.h>

/* Report printf-like ERR, append message for ERR_NO if ERR_NO != 0;
   Each frontend defines its own implementation. */
void
error (int err_no, const char *err, ...)
{
  va_list ap;

  fprintf (stderr, "%s: ", program_name);
  va_start (ap, err);
  vfprintf (stderr, err, ap);
  va_end (ap);
  if (err_no == 0)
    fputc ('\n', stderr);
  else
    fprintf (stderr, ": %s\n", strerror (err_no));
}

/* Set program_name from information in ARGV_0 */
inline static void
set_program_name (const char *argv_0)
{
  if (argv_0 == NULL)
    argv_0 = "cereal_disasm";
  program_name = xstrdup (argv_0);
}

/* Print help and exit */
inline static void attribute__ ((noreturn))
arg_help (void)     
{
  printf ("Usage: %s [file] ... \n"
	  "Loads all files (in Intel HEX format) and outputs disassembling\n"
	  "Report bugs to <" PACKAGE_BUGREPORT ">.\n", program_name);
  exit (EXIT_SUCCESS);
}

/* Print version and exit */
inline static void attribute__ ((noreturn))
arg_version (void)
{
  printf ("Cereal Emulates REALity " PACKAGE_VERSION "\n"
	  "Copyright (C) 2002 Miloslav Trmac <mitr@volny.cz>\n"
	  "You may redistribute copies of cereal under the terms of the GNU "
	  "General\n"
	  "Public License, version 2.  See the file COPYING in the source "
	  "tarball.\n"
	  "cereal comes with NO WARRANTY, to the extent permitted by law.\n");
  exit (EXIT_SUCCESS);
}

int
main (int argc, char *argv[])
{
  uint8_t prog[65536], *end, *p;
  int use_stdin;
  
  set_program_name (argv[0]);
  memset (prog, 0x00, sizeof (prog));
  use_stdin = 1;
  if (argc > 1)
    {
      while (*++argv != NULL)
	{
	  if (strcmp (*argv, "--help") == 0)
	    arg_help ();
	  else if (strcmp (*argv, "--version") == 0)
	    arg_version ();
	  else
	    break;
	}
      if (*argv != NULL && strcmp (*argv, "--") == 0)
	argv++;
      if (*argv != NULL)
	{
	  use_stdin = 0;
	  do
	    {
	      if (load_hex_file (*argv, prog, sizeof (prog)) != 0)
		return EXIT_FAILURE;
	    }
	  while (*++argv != NULL);
	}
    }
  if (use_stdin != 0 && load_hex_stdio ("standard input", stdin, prog,
					sizeof (prog)) != 0)
    return EXIT_FAILURE;
  end = prog;
  for (p = prog; p < prog + ARRAY_SIZE (prog); p++)
    {
      if (*p != 0)
	end = p + 1;
    }
  p = prog;
  while (p < end)
    {
      const char *opcode;
      char buf[64];
      size_t len;
      uint16_t pc;

      pc = p - prog;
      len = i8051_disasm (pc, p, &opcode, buf);
      printf ("0x%04" PRIX16 "\t%s\t%s\n", pc, opcode, buf);
      p += len;
    }
  return EXIT_SUCCESS;
}
